import React, { createContext, useContext, useState } from "react";
import cockpit from "cockpit";
import {
    Alert,
    AlertActionCloseButton,
    AlertGroup,
    CodeBlock,
    CodeBlockCode,
    Page,
    PageSection,
    PageSectionVariants,
    Stack
} from "@patternfly/react-core";
import { ExclamationCircleIcon, SpinnerIcon } from "@patternfly/react-icons";
import { EmptyStatePanel } from "../../pkg/lib/cockpit-components-empty-state.jsx";
import { HelperText, HelperTextItem } from "@patternfly/react-core/dist/esm/components/HelperText";
import { WithDialogs } from "../../pkg/lib/dialogs.jsx";
import { SuperuserButton } from "../../pkg/shell/superuser.jsx";

const _ = cockpit.gettext;

export function ErrorIcon () {
    return (
        <HelperText>
            <HelperTextItem variant='error' hasIcon>
                {_("Error")}
            </HelperTextItem>
        </HelperText>
    );
}

export const ErrorMessage = ({
    loading,
    error
}) => {
    return (
        <Page>
            <PageSection variant={PageSectionVariants.light}>
                <EmptyStatePanel
                    icon={error ? ExclamationCircleIcon : SpinnerIcon}
                    loading={loading}
                    title={error ? _("Something went wrong") : _("Loading. Please wait...")}
                    paragraph={
                        error && (
                            <Stack hasGutter>
                                <div>{_("There was an unexpected error.")}</div>
                                {error === "access-denied" &&
                                    <div>{_("Error: Root permissions required for this page.")}</div>}
                                <CodeBlock>
                                    <CodeBlockCode
                                        id='code-content'
                                    >{error}
                                    </CodeBlockCode>
                                </CodeBlock>
                            </Stack>)
                    }
                />
            </PageSection>
        </Page>
    );
};

const NotificationContext = createContext();

export const useNotifications = () => useContext(NotificationContext);

export function NotificationProvider ({ children }) {
    const [notifications, setNotifications] = useState([]);

    // Function to add a notification
    const addNotification = (title, message, variant = "info") => {
        const newNotification = {
            id: new Date().getTime(),
            variant,
            title,
            message
        };
        setNotifications(prevNotifications => [...prevNotifications, newNotification]);
    };

    // Function to remove a notification
    const removeNotification = (id) => {
        setNotifications(notifications.filter(notification => notification.id !== id));
    };

    return (
        <NotificationContext.Provider value={{ addNotification, removeNotification, notifications }}>
            {children}
        </NotificationContext.Provider>
    );
}

export function Notifications () {
    const { notifications, removeNotification } = useNotifications();

    return (
        <AlertGroup isToast>
            {notifications.map(notification => (
                <Alert
                    key={notification.id}
                    variant={notification.variant}
                    title={notification.title}
                    actionClose={<AlertActionCloseButton onClose={() => removeNotification(notification.id)} />}
                >
                    {notification.message}
                </Alert>
            ))}
        </AlertGroup>
    );
}

export function AdminAccessButton ({ featureName = _("RevPi Configuration") }) {
    return (
        <WithDialogs>
            <EmptyStatePanel
                title={_("Administrative access required")}
                paragraph={cockpit.format(_("Administrative access is required for using $0."), featureName)}
                action={<SuperuserButton />}
            />
        </WithDialogs>
    );
}
