import cockpit from "cockpit";
import React, { useCallback, useEffect, useState } from "react";
import {
    AdminAccessButton,
    ErrorMessage,
    NotificationProvider,
    Notifications,
    useNotifications
} from "./app.components.jsx";
import { Page, PageSection, PageSectionVariants, Grid, GridItem } from "@patternfly/react-core";
import {
    RevPiSwitch,
    RevPiToggle
} from "./common.components.jsx";
import { PictoryCard } from "./features/PictoryCard.jsx";
import { NetworkCard } from "./features/NetworkCard.jsx";
import { NodeRedCard } from "./features/NodeRedCard.jsx";
import { SystemConfigurationCard } from "./features/SystemConfigurationCard.jsx";
import { RunTimeCard } from "./features/RunTimeCard.jsx";
import {
    getRevPiConfigStatus,
    hasLowMemory,
    toggleConfiguration
} from "./revpi-config/RevPiConfigService.jsx";
import { useConfigData } from "./revpi-config/RevPiConfigData.jsx";
import { installEventEmitter } from "./emitters.js";

const _ = cockpit.gettext;

export const ApplicationRoot = ({ children }) => (
    <NotificationProvider>
        <Page>
            <PageSection variant={PageSectionVariants.default}>
                <Notifications />
                {children}
            </PageSection>
        </Page>
    </NotificationProvider>
);

export const Application = () => {
    const [isLoading, setIsLoading] = useState(true);
    const [features, setFeatures] = useState(new Map());
    const [error, setError] = useState(null);

    const { addNotification } = useNotifications();
    const configData = useConfigData();

    const updateFeatureLoadingState = useCallback((id, isLoading) => {
        setFeatures((prev) =>
            new Map(prev).set(id, { ...prev.get(id) || {}, isLoading: !isLoading })
        );
    }, []);

    const setFeatureError = useCallback((id) => {
        setFeatures((prev) => new Map(prev).set(id, {
            ...prev.get(id) || {},
            hasError: true
        }));
    }, []);

    const loadAllFeatures = useCallback(async () => {
        try {
            const featureList = await getRevPiConfigStatus(configData);
            setFeatures(new Map(featureList.map((x) => [x.id, x])));
            setIsLoading(false);
        } catch (error) {
            console.error("Error fetching features:", error);
            setError(error.message || "unknown-error");
            setIsLoading(false);
        }
    }, [configData]);

    useEffect(() => {
        installEventEmitter.addEventListener(
            installEventEmitter.events.finish,
            loadAllFeatures
        );
        return () => {
            installEventEmitter.removeEventListener(installEventEmitter.events.finish, loadAllFeatures);
        };
    }, [loadAllFeatures]);

    const refreshFeatures = useCallback(async () => {
        setError(null);
        setIsLoading(true);
        await loadAllFeatures();
    }, [loadAllFeatures]);

    const handleGuiEnableLogic = useCallback(async () => {
        const lowMemory = await hasLowMemory();
        if (!lowMemory) return;

        const swapId = "dphys-swapfile";
        updateFeatureLoadingState(swapId, isLoading);

        addNotification(
            _("Swap / Page File Enabled for Better Performance"),
            cockpit.format(
                _("Your device has 1GB of RAM or less. To ensure smooth performance in GUI mode, \"$0\" has been automatically enabled. You can disable it if you prefer."),
                _("Swap / page file (recommended for GUI)")
            ),
            "info"
        );

        await toggleConfiguration(swapId, false);
    }, [updateFeatureLoadingState, addNotification, isLoading]);

    const handleSwitchClick = useCallback(async (id, isEnabled, isLoading) => {
        updateFeatureLoadingState(id, isLoading);
        try {
            await toggleConfiguration(id, isEnabled);
            // If "gui" is being enabled, also enable "swap"
            if (id === "gui" && !isEnabled) {
                await handleGuiEnableLogic();
            }
            await loadAllFeatures();
        } catch {
            updateFeatureLoadingState(id, !isLoading);
            setFeatureError(id);
        }
    }, [loadAllFeatures, updateFeatureLoadingState, setFeatureError, handleGuiEnableLogic]);

    const getRevPiSwitch = useCallback((id) => {
        return <RevPiSwitch item={features.get(id)} switchHandler={handleSwitchClick} />;
    }, [features, handleSwitchClick]);

    const getRevPiToggle = useCallback((id) => {
        return <RevPiToggle item={features.get(id)} switchHandler={handleSwitchClick} />;
    }, [features, handleSwitchClick]);

    useEffect(() => {
        const load = async () => {
            setError(null);
            setIsLoading(true);

            const permission = cockpit.permission({ admin: true });
            const handlePermissionChange = async () => {
                if (permission.allowed) {
                    await refreshFeatures();
                } else {
                    setError("access-denied");
                    setIsLoading(false);
                }
            };

            permission.addEventListener("changed", handlePermissionChange);

            // Initial permission check
            await handlePermissionChange();

            return () => {
                permission.removeEventListener("changed", handlePermissionChange);
            };
        };

        load();
    }, [refreshFeatures]);

    if (isLoading || error) {
        if (error === "access-denied") {
            return (
                <AdminAccessButton />
            );
        }
        return <ErrorMessage loading={isLoading} error={error} />;
    }

    return (
        <ApplicationRoot>
            <Grid className='ct-system-overview' hasGutter>
                <GridItem md={4} sm={12}>
                    <PictoryCard />
                </GridItem>

                <GridItem md={4} sm={12}>
                    <NodeRedCard
                        item={features.get("nodered")}
                        getRevPiSwitch={getRevPiSwitch}
                    />
                </GridItem>

                <GridItem md={4} sm={12}>
                    <RunTimeCard
                        getRevPiSwitch={getRevPiSwitch}
                        getRevPiToggle={getRevPiToggle}
                    />
                </GridItem>

                <GridItem md={6} sm={12}>
                    <SystemConfigurationCard
                        getRevPiSwitch={getRevPiSwitch}
                        getRevPiToggle={getRevPiToggle}
                    />
                </GridItem>

                <GridItem md={6} sm={12}>
                    <NetworkCard
                        getRevPiSwitch={getRevPiSwitch}
                        getRevPiToggle={getRevPiToggle}
                    />
                </GridItem>
            </Grid>
        </ApplicationRoot>
    );
};
