import React, { useEffect, useState } from "react";
import cockpit from "cockpit";
// eslint-disable-next-line camelcase
import { install_dialog } from "cockpit-components-install-dialog.jsx";
import {
    Alert,
    Card,
    CardBody,
    CardHeader
} from "@patternfly/react-core";
import { useNotifications } from "../app.components.jsx";
import { Button } from "@patternfly/react-core/dist/esm/components/Button";
import { installEventEmitter } from "../emitters.js";

const _ = cockpit.gettext;

export function InstallFeatureCard ({ featureName, packagesName, children }) {
    const [isInstalled, handleInstallationComplete] =
    useInstallationStatus(packagesName);
    const [globalIsInstalling, setGlobalIsInstalling] = useState(false);
    const { addNotification } = useNotifications();
    const notificationTitle = _("Installation successful");
    const notificationText = _("$0 successfully installed");

    useEffect(() => {
        const handleInstallationStart = () => {
            setGlobalIsInstalling(true);
        };
        const handleInstallationFinish = () => {
            setGlobalIsInstalling(false);
        };

        // Add the event listener
        installEventEmitter.addEventListener(
            installEventEmitter.events.start,
            handleInstallationStart
        );
        installEventEmitter.addEventListener(
            installEventEmitter.events.finish,
            handleInstallationFinish
        );

        // Cleanup the event listener on component unmount
        return () => {
            installEventEmitter.removeEventListener(
                installEventEmitter.events.start,
                handleInstallationStart
            );
            installEventEmitter.removeEventListener(
                installEventEmitter.events.finish,
                handleInstallationFinish
            );
        };
    }, []);

    const handleInstallClick = async () => {
        installEventEmitter.dispatchEvent(installEventEmitter.events.start);
        try {
            await install_dialog(packagesName);
            handleInstallationComplete();
            addNotification(notificationTitle, cockpit.format(notificationText, featureName));
        } catch (error) {
        } finally {
            installEventEmitter.dispatchEvent(installEventEmitter.events.finish);
        }
    };

    if (isInstalled) return <>{children}</>;

    return (
        <Card>
            <CardHeader>
                <Alert
                    variant='info'
                    isInline
                    isPlain
                    title={cockpit.format(_("$0 is not installed."), featureName)}
                />
            </CardHeader>
            <CardBody>
                <Button onClick={handleInstallClick} isDisabled={globalIsInstalling}>
                    {cockpit.format(_("Install $0"), featureName)}
                </Button>
            </CardBody>
        </Card>
    );
}

async function isPackageInstalled (packageName) {
    try {
        await cockpit.spawn([
            "bash",
            "-c",
            `dpkg -l | grep -w '^ii  ${packageName}' `
        ]);
        return true;
    } catch (error) {
        console.info(`Info: ${packageName} is not installed.`, error);
        return false;
    }
}

/**
 *
 * @param packageNames
 * @returns {(boolean|(function(): Promise<void>)|*)[]}
 */
function useInstallationStatus (packageNames) {
    const [isInstalled, setIsInstalled] = useState(false);
    useEffect(() => {
        async function checkInstallation () {
            let installed = true;
            for (const packageName of packageNames) {
                const isPkgInstalled = await isPackageInstalled(packageName);
                installed = installed && isPkgInstalled;
                if (!installed) break;
            }

            setIsInstalled(installed);
        }

        checkInstallation();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []); // Empty dependency array ensures this runs only once on mount
    const handleInstallationComplete = async () => {
        let installed = true;
        for (const packageName of packageNames) {
            const isPkgInstalled = await isPackageInstalled(packageName);
            installed = installed && isPkgInstalled;
            if (!installed) break;
        }
        setIsInstalled(installed);
    };
    return [isInstalled, handleInstallationComplete];
}
