import React, { useEffect, useState } from "react";
import cockpit from "cockpit";
import {
    MenuToggle,
    Select,
    SelectList,
    SelectOption,
    TextInputGroup,
    TextInputGroupMain,
    TextInputGroupUtilities
} from "@patternfly/react-core";
import { Button } from "@patternfly/react-core/dist/esm/components/Button";
import { TimesIcon } from "@patternfly/react-icons";
import { RevPiFlexBase } from "../common.components.jsx";

const _ = cockpit.gettext;

export const WlanCountrySelect = () => {
    const [isLoading, setIsLoading] = useState(true);
    const [hasError, setHasError] = useState(false);
    const [allCountryCodes, setAllCountryCodes] = useState([]);
    const [currentCountryCode, setCurrentCountryCode] = useState("");

    useEffect(() => {
        loadData();
    }, []);

    const loadData = async () => {
        setIsLoading(true);
        try {
            const allCodes = await getAllCountryCodes();
            const currentCode = await getCurrentCountryCode();
            setAllCountryCodes(allCodes);
            setCurrentCountryCode(currentCode);
        } catch (e) {
            console.error(e);
            setHasError(true);
        } finally {
            setIsLoading(false);
        }
    };

    const handleWlanCountrySelect = async (dropdownSelection) => {
        setIsLoading(true);
        try {
            await setNewCountryCode(dropdownSelection);
            setCurrentCountryCode(dropdownSelection);
        } catch (e) {
            console.error("Error while handling new wlan country", e);
            setHasError(true);
        } finally {
            setIsLoading(false);
        }
    };

    const getHelpText = () => (
        <div className='help-text'>
            <p>{_("Choosing the right country setting for WiFi helps ensure your connection works smoothly.")}</p>

            <p><strong>{_("Why is this important?")}</strong></p>
            <ul>
                <li>{_("Different countries allow different WiFi channels and signal strengths.")}</li>
                <li>{_("Setting the correct country helps prevent connection issues and improves network reliability.")}</li>
            </ul>

            <p><strong>{_("What should I choose?")}</strong></p>
            <ul>
                <li>{_("Select the country where you are using the device.")}</li>
            </ul>

            <p><strong>{_("What happens if I don’t set it?")}</strong></p>
            <ul>
                <li>{_("WiFi might not work as expected, or some networks may not appear.")}</li>
                <li>{_("Your connection could be slower or less stable.")}</li>
                <li>{_("In some cases, using the wrong setting might interfere with other devices.")}</li>
            </ul>
        </div>
    );

    return (
        <RevPiFlexBase
            item={{
                title: _("WLAN country"),
                rebootRequired: true,
                isLoading,
                hasError,
                helpText: getHelpText()
            }}
        >
            {allCountryCodes.length > 0 && (
                <SearchSelect
                    options={allCountryCodes}
                    status={currentCountryCode}
                    handleSelect={handleWlanCountrySelect}
                />
            )}
        </RevPiFlexBase>
    );
};

export class WlanCountrySelector extends React.Component {
    constructor () {
        super();
        this.state = {
            isLoading: true,
            hasError: false,
            allCountryCodes: [],
            currentCountryCode: ""
        };
    }

    componentDidMount () {
        this.loadData();
    }

    async loadData () {
        this.setState({ isLoading: true });
        try {
            const allCountryCodes = await getAllCountryCodes();
            const currentCountryCode = await getCurrentCountryCode();
            this.setState({
                allCountryCodes,
                currentCountryCode
            });
        } catch (e) {
            console.error(e);
            this.setState({ hasError: true });
            this.setState({ isLoading: false });
        } finally {
            this.setState({ isLoading: false });
        }
    }

    async handleWlanCountrySelect (dropdownSelection) {
        this.setState({ isLoading: true });
        try {
            await setNewCountryCode(dropdownSelection);
            this.setState({ currentCountryCode: dropdownSelection });
        } catch (e) {
            console.error("Error while handling new wlan country", e);
            this.setState({
                hasError: true
            });
        } finally {
            this.setState({ isLoading: false });
        }
    }

    getHelpText = () => {
        return (
            <div className='help-text'>
                <p>{_("Choosing the right country setting for WiFi helps ensure your connection works smoothly.")}</p>

                <p><strong>{_("Why is this important?")}</strong></p>
                <ul>
                    <li>{_("Different countries allow different WiFi channels and signal strengths.")}</li>
                    <li>{_("Setting the correct country helps prevent connection issues and improves network reliability.")}</li>
                </ul>

                <p><strong>{_("What should I choose?")}</strong></p>
                <ul>
                    <li>{_("Select the country where you are using the device.")}</li>
                </ul>

                <p><strong>{_("What happens if I don’t set it?")}</strong></p>
                <ul>
                    <li>{_("WiFi might not work as expected, or some networks may not appear.")}</li>
                    <li>{_("Your connection could be slower or less stable.")}</li>
                    <li>{_("In some cases, using the wrong setting might interfere with other devices.")}</li>
                </ul>
            </div>
        );
    };

    render () {
        return (
            <RevPiFlexBase
                item={{
                    title: _("WLAN country"),
                    rebootRequired: true,
                    isLoading: this.state.isLoading,
                    hasError: this.state.hasError,
                    helpText: this.getHelpText()
                }}
            >
                {
                    this.state.allCountryCodes.length > 0 &&
                    <SearchSelect
                        options={this.state.allCountryCodes}
                        status={this.state.currentCountryCode}
                        handleSelect={(selection) => this.handleWlanCountrySelect(selection)}
                    />
                }
            </RevPiFlexBase>
        );
    }
}

async function getAllCountryCodes () {
    try {
        const fileContent = await cockpit.file("/usr/share/zoneinfo/iso3166.tab")
            .read();
        const countryCodes = fileContent.split("\n")
            .map(line => {
                const entries = line.split("\t");
                return entries[0] + " " + entries[1];
            })
            .filter(function (entry) {
                return entry !== "" && entry[0] !== "#" && entry.trim() !== "undefined";
            });
        return [...new Set(countryCodes.slice(countryCodes.indexOf("#code") + 1))];
    } catch (error) {
        console.error("An error occurred while loading wlan country codes.", error);
        throw new Error("An error occurred while loading country codes: " + error);
    }
}

async function getCurrentCountryCode () {
    try {
        const code = await cockpit.spawn(["raspi-config", "nonint", "get_wifi_country"], { superuser: "required" });
        return code;
    } catch (e) {
        console.log("raspi-config exited with code 1; setting country code to an empty string.");
        return "";
    }
}

async function setNewCountryCode (newCode) {
    try {
        await cockpit.spawn(["raspi-config", "nonint", "do_wifi_country", newCode], { superuser: "require" });
    } catch (e) {
        console.error("Could not set the new wlan country.", e);
        throw new Error("Could not set the new wlan country.");
    }
}

export const SearchSelect = (props) => {
    const extractCountryCode = (option) => option.split(" ")[0];
    const getFullCountryString = (code) => props.options.find(c => c.includes(code));
    // initial values
    const initialSelectOptions = props.options.map((e) => ({
        value: extractCountryCode(e),
        children: e
    }));
    const initialSelection = props.options.find(option => extractCountryCode(option) === props.status.trim());

    // state
    const [isOpen, setIsOpen] = React.useState(false);
    const [selected, setSelected] = React.useState(initialSelection);
    const [inputValue, setInputValue] = React.useState(initialSelection);
    const [filterValue, setFilterValue] = React.useState("");
    const [selectOptions, setSelectOptions] = React.useState(initialSelectOptions);
    const [focusedItemIndex, setFocusedItemIndex] = React.useState(null);
    const [activeItem, setActiveItem] = React.useState(null);
    const textInputRef = React.useRef();
    React.useEffect(() => {
        let newSelectOptions = initialSelectOptions;
        if (filterValue) {
            newSelectOptions = initialSelectOptions.filter(menuItem => String(menuItem.children)
                .toLowerCase()
                .includes(filterValue.toLowerCase()));
            if (!newSelectOptions.length) {
                newSelectOptions = [{
                    isDisabled: false,
                    children: `No results found for "${filterValue}"`,
                    value: "no results"
                }];
            }
            if (!isOpen) {
                setIsOpen(true);
            }
        }
        setSelectOptions(newSelectOptions);
        setActiveItem(null);
        setFocusedItemIndex(null);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [filterValue]);
    const onToggleClick = () => {
        setIsOpen(!isOpen);
    };
    const handleSelection = (value) => {
        if (props.options.find(o => o.includes(value)) && value.trim() !== "") {
            const fullCountryString = getFullCountryString(value);
            setInputValue(fullCountryString);
            setSelected(fullCountryString);
            props.handleSelect(value);
        }
    };
    const onSelect = (_event, value) => {
        if (value && value !== "no results") {
            setFilterValue("");
            handleSelection(value);
        }
        setIsOpen(false);
        setFocusedItemIndex(null);
        setActiveItem(null);
    };
    const onTextInputChange = (_event, value) => {
        setInputValue(value);
        setFilterValue(value);
    };
    const handleMenuArrowKeys = key => {
        let indexToFocus;
        if (isOpen) {
            if (key === "ArrowUp") {
                if (focusedItemIndex === null || focusedItemIndex === 0) {
                    indexToFocus = selectOptions.length - 1;
                } else {
                    indexToFocus = focusedItemIndex - 1;
                }
            }
            if (key === "ArrowDown") {
                if (focusedItemIndex === null || focusedItemIndex === selectOptions.length - 1) {
                    indexToFocus = 0;
                } else {
                    indexToFocus = focusedItemIndex + 1;
                }
            }
            setFocusedItemIndex(indexToFocus);
            const focusedItem = selectOptions.filter(option => !option.isDisabled)[indexToFocus];
            setActiveItem(`select-typeahead-${focusedItem.value.replace(" ", "-")}`);
        }
    };
    const onInputKeyDown = event => {
        const enabledMenuItems = selectOptions.filter(option => !option.isDisabled);
        const [firstMenuItem] = enabledMenuItems;
        const focusedItem = focusedItemIndex ? enabledMenuItems[focusedItemIndex] : firstMenuItem;
        switch (event.key) {
        case "Enter":
            if (isOpen && focusedItem.value !== "no results") {
                setInputValue(String(focusedItem.children));
                setFilterValue("");
                handleSelection(String(focusedItem.value));
            }
            setIsOpen(prevIsOpen => !prevIsOpen);
            setFocusedItemIndex(null);
            setActiveItem(null);
            break;
        case "Tab":
        case "Escape":
            setIsOpen(false);
            setActiveItem(null);
            break;
        case "ArrowUp":
        case "ArrowDown":
            event.preventDefault();
            handleMenuArrowKeys(event.key);
            break;
        }
    };
    const toggle = toggleRef => (
        <MenuToggle
            ref={toggleRef} variant='typeahead' onClick={onToggleClick}
            isExpanded={isOpen} isFullWidth
        >
            <TextInputGroup isPlain>
                <TextInputGroupMain
                    value={inputValue}
                    onClick={onToggleClick}
                    onChange={onTextInputChange}
                    onKeyDown={onInputKeyDown}
                    id='typeahead-select-input'
                    autoComplete='off'
                    innerRef={textInputRef}
                    placeholder={_("Select country")} {...(activeItem && {
                        "aria-activedescendant": activeItem
                    })}
                    role='combobox'
                    isExpanded={isOpen}
                    aria-controls='select-typeahead-listbox'
                />

                <TextInputGroupUtilities>
                    {!!inputValue &&
                    <Button
                        variant='plain' onClick={() => {
                            handleSelection("");
                            setInputValue("");
                            setFilterValue("");
                            textInputRef?.current?.focus();
                        }} aria-label='Clear input value'
                    >
                        <TimesIcon aria-hidden />
                    </Button>}
                </TextInputGroupUtilities>
            </TextInputGroup>
        </MenuToggle>
    );
    return (
        <Select
            id='typeahead-select' isOpen={isOpen} selected={selected} onSelect={onSelect} onOpenChange={() => {
                setIsOpen(false);
            }} toggle={toggle}
        >
            <SelectList
                id='select-typeahead-listbox' style={{
                    maxHeight: "250px",
                    overflow: "auto"
                }}
            >
                {selectOptions.map((option, index) =>
                    <SelectOption
                        key={option.value || option.children}
                        isFocused={focusedItemIndex === index}
                        className={option.className}
                        onClick={() => handleSelection(option.value)}
                        id={`select-typeahead-${option.value.replace(" ", "-")}`} {...option}
                        ref={null}
                    />)}
            </SelectList>
        </Select>
    );
};
